/* crash-pipe: handle core file passed from stdin
 *
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Łukasz Stelmach <l.stelmach@samsung.com>
 */
#define _GNU_SOURCE

#define REG_FP 11
#define REG_IP 12
#define REG_SP 13
#define REG_LR 14
#define REG_PC 15
#define REG_SPSR 16

#include <stdio.h>
#include <sys/procfs.h>

unsigned long arch_get_bp(elf_gregset_t* r)
{
	struct user_regs *regs = (struct user_regs *)r;
	return regs->uregs[REG_FP];
}

unsigned long arch_get_ip(elf_gregset_t* r)
{
	struct user_regs *regs = (struct user_regs *)r;
	return regs->uregs[REG_IP];
}

unsigned long arch_get_sp(elf_gregset_t* r)
{
	struct user_regs *regs = (struct user_regs *)r;
	return regs->uregs[REG_SP];
}

void arch_dump_registers(elf_gregset_t* r)
{
	struct user_regs *regs = (struct user_regs*)r;
	printf("\nRegister Information\n");
	printf("pc : [<%08lx>]    lr : [<%08lx>]    psr: %08lx\n",
	       regs->uregs[REG_PC], regs->uregs[REG_LR],
	       regs->uregs[REG_SPSR]
	       );
	printf("sp : %08lx  ip : %08lx  fp : %08lx\n",
	       regs->uregs[REG_SP], regs->uregs[REG_IP],
	       regs->uregs[REG_FP]
	       );
	printf("r10: %08lx  r9 : %08lx  r8 : %08lx\n",
	       regs->uregs[10], regs->uregs[9],
	       regs->uregs[8]
	       );
	printf("r7 : %08lx  r6 : %08lx  r5 : %08lx  r4 : %08lx\n",
	       regs->uregs[7], regs->uregs[6],
	       regs->uregs[5], regs->uregs[4]
	       );
	printf("r3 : %08lx  r2 : %08lx  r1 : %08lx  r0 : %08lx\n",
	       regs->uregs[3], regs->uregs[2],
	       regs->uregs[1], regs->uregs[0]
	       );
}
