/* crash-pipe: handle core file passed from stdin
 *
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Author: Łukasz Stelmach <l.stelmach@samsung.com>
 */
#define _GNU_SOURCE

#include <sys/procfs.h>
#include <stdio.h>

unsigned long arch_get_bp(elf_gregset_t* r)
{
	struct user_regs_struct *regs = (struct user_regs_struct *)r;
	return regs->rbp;
}

unsigned long arch_get_ip(elf_gregset_t* r)
{
	struct user_regs_struct *regs = (struct user_regs_struct *)r;
	return regs->rip;
}

unsigned long arch_get_sp(elf_gregset_t* r)
{
	struct user_regs_struct *regs = (struct user_regs_struct *)r;
	return regs->rsp;
}

void arch_dump_registers(elf_gregset_t* r)
{
	struct user_regs_struct *regs = (struct user_regs_struct *)r;
#define _PRINT_REGISTERS(a,b,c)						\
	printf("%3s: %016lx %3s: %016lx %3s: %016lx\n",			\
	       #a, regs->a,						\
	       #b, regs->b,						\
	       #c, regs->c)
	printf("\nRegister Information\n");
#ifdef __x86_64__
	printf("rip: %04lx:[<%016lx>]\n",
		regs->cs & 0xffff,
		regs->rip);
	printf("rsp: %04lx:%016lx eflags: %08lx\n",
		regs->ss,
		regs->rsp,
		regs->eflags);
	_PRINT_REGISTERS(rax, rbx, rcx);
	_PRINT_REGISTERS(rdx, rsi, rdi);
	_PRINT_REGISTERS(rbp, r8, r9);
	_PRINT_REGISTERS(r10, r11, r12);
	_PRINT_REGISTERS(r13, r14, r15);
#else
	printf("Unsupported architecture\n")
#endif
	printf("\n");
#undef _PRINT_REGISTERS
}
